@@ -194,22 +194,20 @@ def text(string, x, y, color, tf=None):
         yield
 
 
-def image(tex_id, w, h, tf):
-    """ Draw an image with the given width and height.
+def image(tex_id, x, y, width, height, uv_a=(0, 0), uv_b=(1, 1), tf=None):
+    """ Draw an image with the given origin (x, y), width and height.
 
-    This is a raw drawing function. Use `concur.extra_widgets.image.image` instead if you want an image widget.
+    This is a raw drawing function. Use `concur.extra_widgets.image.image` instead
+    if you want an image widget. Note that OpenGL textures may be rendered incorrectly
+    if width or height isn't divisible by 4.
     """
-    x0, y0, x1, y1 = 0, 0, w, h
+    p1, p2 = [x, y], [x + width, y + height]
     if tf is not None:
-        [x0, y0], [x1, y1] = tf.transform(np.array([[x0, y0], [x1, y1]]))
+        p1, p2 = tf.transform(np.array([p1, p2]))
+
     draw_list = imgui.get_window_draw_list()
-    l, t, r, b = tf.view_s
-    a_s = tf.view_s[:2]
-    b_s = tf.view_s[2:]
-    a_i = tf.view_c[0] / w, tf.view_c[1] / h
-    b_i = tf.view_c[2] / w, tf.view_c[3] / h
     while True:
-        draw_list.add_image(tex_id, tuple(a_s), tuple(b_s), tuple(a_i), tuple(b_i))
+        draw_list.add_image(tex_id, tuple(p1), tuple(p2), uv_a, uv_b)
         yield
 
 
@@ -267,6 +265,7 @@ def scatter(pts, color, marker, marker_size=10, thickness=1, tf=None):
     `"x"`  | cross
     `"+"`  | plus sign
     `"o"`  | non-filled circle
+    `"s"`  | non-filled square
     """
     if len(pts) == 0:
         pts = pts.reshape(-1, 2)
@@ -307,5 +306,13 @@ def scatter(pts, color, marker, marker_size=10, thickness=1, tf=None):
         polys[...,1] = np.cos(t) * r
         polys += pts.reshape(-1, 1, 2)
         return polylines(polys, color, True, thickness)
+    elif marker in ['s', 'S']:
+        r = marker_size / 2
+        t = np.pi/4 + np.linspace(0, np.pi * 2, 4, endpoint=False)
+        polys = np.empty((len(pts), 4, 2))
+        polys[...,0] = np.sin(t) * r
+        polys[...,1] = np.cos(t) * r
+        polys += pts.reshape(-1, 1, 2)
+        return polylines(polys, color, True, thickness)
     else:
         raise ValueError('Invalid marker')
