@@ -28,7 +28,9 @@ class PatchedGlfwRenderer(GlfwRenderer):
     def keyboard_callback(self, window, key, scancode, action, mods):
         if key < 0x100:
             # Translate characters to the correct keyboard layout.
-            key = ord(glfw.get_key_name(key, 0).upper())
+            key_name = glfw.get_key_name(key, 0)
+            if key_name is not None:
+                key = ord(key_name.upper())
         super(PatchedGlfwRenderer, self).keyboard_callback(window, key, scancode, action, mods)
 
 
@@ -106,6 +108,7 @@ def main(name, widget, width, height, fps=60, save_screencast=None, screencast_f
 
     window = create_window(name, width, height)
     impl = PatchedGlfwRenderer(window)
+    impl.refresh_font_texture() # Refresh the font texture in case user changed it
 
     ## Using this feels significantly choppier than sleeping manually. TODO: investigate & fix
     # glfw.swap_interval(-1)
@@ -142,6 +145,8 @@ def main(name, widget, width, height, fps=60, save_screencast=None, screencast_f
             raise
 
         imgui.end()
+        gl.glClearColor(0.5, 0.5, 0.5, 1)
+        gl.glClear(gl.GL_COLOR_BUFFER_BIT)
         imgui.render()
 
         if save_screencast:
