@@ -50,6 +50,8 @@ def window(title: str,
             imgui.set_next_window_position(*position)
         if size is not None:
             imgui.set_next_window_size(*size)
+        else:
+            imgui.set_next_window_size(400, 300, imgui.FIRST_USE_EVER)
         expanded, opened = imgui.begin(title, flags=flags)
         try:
             if expanded and opened:
@@ -87,6 +89,24 @@ def collapsing_header(text, widget, open=True):
         yield
 
 
+def tree_node(text, widget, open=True):
+    """Display a collapsible tree node. It can be open or closed by default (parameter `open`).
+
+    Tree node content has left offset, unlike `collapsing_header`.
+    """
+    while True:
+        expanded = imgui.tree_node(text, flags=open and imgui.TREE_NODE_DEFAULT_OPEN)
+        try:
+            if expanded:
+                next(widget)
+        except StopIteration as e:
+            return e.value
+        finally:
+            if expanded:
+                imgui.tree_pop()
+        yield
+
+
 def button(label, tag=None):
     """ Button. Returns `(label, None)` on click, or `(tag, None)` if tag is specified. """
     while not imgui.button(label):
@@ -152,6 +172,19 @@ def input_text(name, value, buffer_length=255, tag=None):
             yield
 
 
+def input_text_multiline(name, value, buffer_length=4000, width=0, height=0, flags=0, tag=None):
+    """ Multiline text input.
+
+    Flags are [defined by PyImGui](https://pyimgui.readthedocs.io/en/latest/guide/inputtext-flags.html#inputtext-flag-options).
+    """
+    while True:
+        changed, new_value = imgui.input_text_multiline(name, value, buffer_length, width, height, flags)
+        if changed:
+            return (name if tag is None else tag), new_value
+        else:
+            yield
+
+
 def key_press(name, key_index, ctrl=False, shift=False, alt=False, super=False, repeat=True):
     """ Invisible widget that waits for a given key to be pressed.
 
@@ -288,10 +321,44 @@ def same_line():
     return lift(imgui.same_line)
 
 
+def font(font_, widget):
+    """ Render `widget` with a given font.
+
+    The easiest way to create `font_` is probably to call this before the call to
+    `concur.integrations.glfw.main`:
+
+    ```python
+    imgui.create_context()
+    font = imgui.get_io().fonts.add_font_from_file_ttf("font_file.ttf", 16)
+    ```
+
+    See the [font guide in PyImGui](https://pyimgui.readthedocs.io/en/latest/guide/using-fonts.html)
+    for more details.
+    """
+    while True:
+        imgui.push_font(font_)
+        try:
+            next(widget)
+        except StopIteration as e:
+            return e.value
+        finally:
+            imgui.pop_font()
+        yield
+
+
 def text(s):
     """ Passive text display widget. """
     return lift(imgui.text, s)
 
+def text_wrapped(s):
+    """ Word wrapping text display widget. Recommended for long chunks of text. """
+    def f():
+        imgui.push_text_wrap_pos()
+        imgui.text(s)
+        imgui.pop_text_wrap_pos()
+    return lift(f)
+
+
 def text_colored(s, color):
     """ Passive colored text display widget. """
     r, g, b, a = color_to_rgba_tuple(color)
